@@ -1,5 +1,5 @@
 package JSON::Lines;
-use 5.006; use strict; use warnings; our $VERSION = '1.09';
+use 5.006; use strict; use warnings; our $VERSION = '1.10';
 use Cpanel::JSON::XS; use base 'Import::Export';
 
 our ($JSON, $LINES, %EX);
@@ -32,6 +32,48 @@ sub new {
 	$self;
 }
 
+sub pretty {
+	my ($self, $val) = @_;
+	return $JSON->get_indent if @_ == 1;
+	$JSON->pretty($val);
+	return $self;
+}
+
+sub canonical {
+	my ($self, $val) = @_;
+	return $JSON->get_canonical if @_ == 1;
+	$JSON->canonical($val);
+	return $self;
+}
+
+sub utf8 {
+	my ($self, $val) = @_;
+	return $JSON->get_utf8 if @_ == 1;
+	$JSON->utf8($val);
+	return $self;
+}
+
+sub parse_headers {
+	my ($self, $val) = @_;
+	return $self->{parse_headers} if @_ == 1;
+	$self->{parse_headers} = $val;
+	return $self;
+}
+
+sub error_cb {
+	my ($self, $val) = @_;
+	return $self->{error_cb} if @_ == 1;
+	$self->{error_cb} = $val;
+	return $self;
+}
+
+sub success_cb {
+	my ($self, $val) = @_;
+	return $self->{success_cb} if @_ == 1;
+	$self->{success_cb} = $val;
+	return $self;
+}
+
 sub encode {
 	my ($self, @data) = (shift, scalar @_ == 1 ? @{$_[0]} : @_);
 	@data = $self->_parse_headers(@data) 
@@ -114,9 +156,35 @@ sub get_line {
 	return $self->_decode_line($line);
 }
 
+sub get_line_at {
+	my ($self, $fh, $index, $seek) = @_;
+	my $fh_id = fileno($fh);
+	$self->{_line_pos} //= {};
+	if ($seek) {
+		seek $fh, 0, 0;
+		$self->{_line_pos}{$fh_id} = 0;
+	}
+	$self->{_line_pos}{$fh_id} //= 0;
+	if ($index < $self->{_line_pos}{$fh_id}) {
+		seek $fh, 0, 0;
+		$self->{_line_pos}{$fh_id} = 0;
+	}
+	while ($self->{_line_pos}{$fh_id} < $index && !eof($fh)) {
+		<$fh>;
+		$self->{_line_pos}{$fh_id}++;
+	}
+	return undef if eof($fh);
+	my $line = '';
+	while ($line !~ m/^$LINES/ && !eof($fh)) {
+		$line .= <$fh>;
+		$self->{_line_pos}{$fh_id}++;
+	}
+	return undef unless $line =~ m/^$LINES/;
+	return $self->_decode_line($line);
+}
+
 sub get_subset {
 	my ($self, $fh, $offset, $length, $out_file) = @_;
-
 	my $in_fh = 1;
 	if (ref $fh ne 'GLOB') {
 		my $file = $fh;
@@ -124,7 +192,6 @@ sub get_subset {
 		$fh = $ffh;
 		$in_fh = 0;
 	}
-
 	seek $fh, 0, 0;
 	if ($offset) {
 		<$fh> for ( 1 .. $offset );
@@ -139,25 +206,45 @@ sub get_subset {
 			$line = "";
 		}
 	}
-
 	if (!$in_fh) {
 		close $fh;
 	}
-
 	if ($out_file) {
 		open my $cfh, '>', $out_file or die "cannot open file for writing: $!";
 		print $cfh join "", @lines;
 		close $cfh;
 		return 1;
 	}
-
-	return [ 
-		map { 
+	return [
+		map {
 			$self->_decode_line($_);
 		} @lines
 	];
 }
 
+sub group_lines {
+	my ($self, $fh, $key) = @_;
+	die "Key must be provided for grouping" unless defined $key;
+	seek $fh, 0, 0;
+	my (%groups, $line_num, $line);
+	($line_num, $line) = (0, '');
+	while (!eof($fh)) {
+		my $start_line = $line_num;
+		while ($line !~ m/^$LINES/ && !eof($fh)) {
+			$line .= <$fh>;
+			$line_num++;
+		}
+		last unless $line =~ m/^$LINES/;
+		my $data = $self->_decode_line($line);
+		$line = '';
+		my $value = ref $key eq 'CODE'
+			? do { local $_ = $data; $key->($data) }
+			: $data->{$key};
+		push @{ $groups{$value} }, $start_line;
+	}
+	return \%groups;
+}
+
 sub _decode_line {
 	my ($self, $line) = @_;
 	my $struct = eval { $JSON->decode($line) };
@@ -233,7 +320,7 @@ JSON::Lines - Parse JSONLines with perl.
 
 =head1 VERSION
 
-Version 1.09
+Version 1.10
 
 =cut
 
@@ -369,6 +456,18 @@ Pretty print.
 
 Parse the first line of the stream as headers.
 
+All of the above options can also be get/set dynamically via accessor methods:
+
+	# Get current value
+	my $is_pretty = $jsonl->pretty;
+
+	# Set value (returns $self for chaining)
+	$jsonl->pretty(1)->canonical(1);
+
+	# Set callbacks
+	$jsonl->error_cb(sub { warn "Error: $_[1]" });
+	$jsonl->success_cb(sub { print "Processed: $_[1]" });
+
 =head2 encode
 
 Encode a perl struct into a json lines string.
@@ -431,6 +530,29 @@ Get a subset of JSON lines, optionally pass a file to write to.
 
 	$jsonl->get_subset($file, $offset, $length, $out_file);
 
+=head2 get_line_at
+
+Get a single record at a specific 0-based line index.
+
+	my $record = $jsonl->get_line_at($fh, 5, 1);  # seek to beginning first
+	my $record = $jsonl->get_line_at($fh, 5);     # no seek, read from current position
+
+Returns undef if the index is beyond the end of the file. Works with indices returned by group_lines.
+
+=head2 group_lines
+
+Group lines in a JSONL file by a key value, returning a hash of line indices.
+
+	open my $fh, '<', $file or die $!;
+	my $groups = $jsonl->group_lines($fh, 'category');
+	# Returns: { 'cat1' => [0, 2, 4], 'cat2' => [1, 3] }
+
+	# With a coderef for complex grouping:
+	my $groups = $jsonl->group_lines($fh, sub { $_->{nested}{key} });
+	close $fh;
+
+The returned indices are 0-based and compatible with get_subset offsets.
+
 =head1 AUTHOR
 
 LNATION, C<< <thisusedtobeanemail at gmail.com> >>
